# GOV.UK Mini Environment Admin
# Copyright © 2018  Christopher Baines <mail@cbaines.net>
#
# This file is part of the GOV.UK Mini Environment Admin.
#
# The GOV.UK Mini Environment Admin is free software: you can
# redistribute it and/or modify it under the terms of the GNU Affero
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later
# version.
#
# The GOV.UK Mini Environment Admin is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with the GOV.UK Mini Environment Admin.  If not, see
# <http://www.gnu.org/licenses/>.

class GovukGuix::FetchRevisionJob < Que::Job
  extend EnqueuedJobs
  include ::ShellUtils

  @retry_interval = 30

  def run(commit_hash, options = {})
    backend_type_and_id = options[:backend_type_and_id]
    if backend_type_and_id
      remote_host = Backends.find_by_type_and_id(
        *backend_type_and_id.split('=')
      ).build_remote_host
    end

    sha = fetch_and_checkout(commit_hash, remote_host)

    command = [
      "#{@repository_directory}/guix-pre-inst-env",
      'guix',
      'build',
      '-e',
      '(begin (use-modules (gds packages govuk)) (current-govuk-guix))'
    ]

    output = run_command(*command, run_remotely_on_host: remote_host)

    store_path = output.last.strip
    logger.debug(self.class) { "store_path: #{store_path}" }

    GovukGuix::Revision.create(
      commit_hash: sha,
      store_path: store_path,
      available_services: available_services(store_path)
    )
  end

  def fetch_and_checkout(commit_hash, remote_host)
    if remote_host
      @repository_directory = '/tmp/govuk-guix'

      run_command(
        "/bin/sh",
        "-c",
        "if cd '#{@repository_directory}';"\
        "then git fetch;"\
        "else "\
        "git clone #{self.class.repository_remote_location} '#{@repository_directory}';"\
        "fi",
        run_remotely_on_host: remote_host
      )

      sha = run_command(
        "/bin/sh",
        "-c",
        "cd '#{@repository_directory}';"\
        "git rev-list -n 1 '#{commit_hash}'",
        run_remotely_on_host: remote_host
      ).last.strip

      run_command(
        "/bin/sh",
        "-c",
        "cd '#{@repository_directory}';"\
        "git checkout #{sha}",
        run_remotely_on_host: remote_host
      )
    else
      require 'git'

      @repository_directory = 'tmp/cache/govuk-guix'

      repository = if File.exist? @repository_directory
                     Git.open(@repository_directory, log: Rails.logger)
                   else
                     Git.clone(
                       self.class.repository_remote_location,
                       @repository_directory
                     )
                   end

      repository.fetch

      sha = repository.object(commit_hash).sha

      repository.checkout(sha)
    end

    return sha
  end

  def available_services(store_path)
    command_output = run_command(
      "#{store_path}/bin/govuk",
      "system",
      "available-services",
      "--json"
    )

    json_start = command_output.find_index { |line| line.starts_with?('[') }
    json_output = command_output[json_start..command_output.length].join

    JSON.parse(json_output)
  end

  def self.repository_remote_location
    ENV.fetch(
      'GOVUK_GUIX_REPOSITORY_REMOTE',
      'https://github.com/alphagov/govuk-guix.git'
    )
  end
end
